/*
 * Created on Nov 1, 2004
 * Copyright (C) Andrea Schweer, 2004
 *
 * This file is part of the Greenstone Alerting Service.
 * Refer to the COPYING file in the base directory of this package
 * for licensing information.
 */
package org.greenstone.gsdlas.profiles;

import java.util.*;
import java.util.Map;
import java.util.TreeMap;


/**
 * @author schweer
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public abstract class PredicateFactory {

    private static int id = 0; 
    
    // documentID -> IdEqualsPredicate referring to it
    private static Map documentIdEqualsPredicates = new TreeMap();
    // collectionID -> IdEqualsPredicate referring to it
    private static Map collectionIdEqualsPredicates = new TreeMap();
    
    // substring for collection_name -> SubstringMatchPredicate referring to it
    private static Map collectionNameMatchPredicates = new TreeMap();
    
    // query -> QueryPredicates referring to it
    private static Map documentTitleQueryPredicates = new TreeMap();
    private static Map documentContentQueryPredicates = new TreeMap();
    
    
    /**
     * @param key
     * @param value
     * @return
     * @throws ParseException
     */
    public static Predicate createPredicate(String key, String value)
            throws ParseException {
        Predicate result = null;
        if (key.equals("type")) {
            return result;
        } else if (key.endsWith("ID")) {
            result = createIdEqualsPredicate(key, value);
        } else if (key.startsWith("document")) {
            result = createQueryPredicate(key, value);
        } else if (key.endsWith("_name")) {
            result = createSubstringMatchPredicate(key, value);
        } else {
            throw new ParseException("value " + value + " for field " + key
                    + " does not lead to a parseable predicate");
        }
        result.setID(id++);
        return result;
    }

    /**
     * @param key
     * @param value
     * @return
     */
    private static SubstringMatchPredicate createSubstringMatchPredicate(String key, String value) {
        SubstringMatchPredicate predicate = null;
        if (key.equals("collection_name")) {
            if (collectionNameMatchPredicates.containsKey(value)) {
                predicate = (SubstringMatchPredicate) collectionNameMatchPredicates.get(value);
            } else {
                predicate = new SubstringMatchPredicate(key, value);
                collectionNameMatchPredicates.put(value, predicate);
            }
        } // TODO other fields than collection_name
        return predicate;
    }

    /**
     * @param key
     * @param value
     * @return
     */
    private static IdEqualsPredicate createIdEqualsPredicate(String key, String value) {
        IdEqualsPredicate predicate = null;
        if (key.equals("documentID")) {
            if (documentIdEqualsPredicates.containsKey(value)) {
                predicate = (IdEqualsPredicate) (documentIdEqualsPredicates.get(value));
            } else {
                predicate = new IdEqualsPredicate(key, value);
                documentIdEqualsPredicates.put(value, predicate);
            }
        } else if (key.equals("collectionID")) {
            if (collectionIdEqualsPredicates.containsKey(value)) {
                predicate = (IdEqualsPredicate) collectionIdEqualsPredicates.get(value);
            } else {
                predicate = new IdEqualsPredicate(key, value);
                collectionIdEqualsPredicates.put(value, predicate);
            }
        } // TODO other IDs
        return predicate;
    }

    /**
     * @param field
     * @param query
     * @return
     */
    private static QueryPredicate createQueryPredicate(String field,
            String query) {
        QueryPredicate predicate = new QueryPredicate(field, query);
        if (field.equals("document_content")) {
            documentContentQueryPredicates.put(query, predicate);
        } else if (field.equals("document_title")) {
            documentTitleQueryPredicates.put(query, predicate);
        }
        return predicate;
    }

    public static Collection getAllQueryPredicates() {
        Set result = new TreeSet();
        result.addAll(documentTitleQueryPredicates.values());
        result.addAll(documentContentQueryPredicates.values());
        return Collections.unmodifiableSet(result);
    }
    
    public static Collection getQueryPredicates(String field) {
        Set result = new TreeSet();
        if (field.equals("document_title")) {
            result.addAll(documentTitleQueryPredicates.values());
        } else if (field.equals("document_content")) {
            result.addAll(documentContentQueryPredicates.values());
        }
        return Collections.unmodifiableSet(result);
    }
    
    public static Collection getAllSubstringMatchPredicates() {
        return Collections.unmodifiableCollection(collectionNameMatchPredicates.values());
    }
    
}